@@ -1,11 +1,11 @@
 #![allow(clippy::from_iter_instead_of_collect)]
 
-use std::collections::{BTreeMap, BTreeSet, HashMap};
+use std::collections::BTreeMap;
 use std::iter::FromIterator;
 
 use sn_dbc::{
-    bls_dkg_id, Amount, AmountSecrets, Dbc, DbcContent, Error, Mint, ReissueRequest,
-    SimpleKeyManager, SimpleSigner, SimpleSpendBook,
+    bls_dkg_id, Amount, AmountSecrets, Dbc, DbcContent, Error, KeyManager, MintNode,
+    ReissueRequestBuilder, SimpleKeyManager, SimpleSigner, SpentProof, SpentProofShare,
 };
 
 use criterion::{black_box, criterion_group, criterion_main, Criterion};
@@ -19,13 +19,7 @@ fn decrypt_amount_secrets(
     dbcc.amount_secrets_by_secret_key_shares(&owner.public_key_set, &shares)
 }
 
-fn genesis(
-    amount: Amount,
-) -> (
-    Mint<SimpleKeyManager, SimpleSpendBook>,
-    bls_dkg::outcome::Outcome,
-    Dbc,
-) {
+fn genesis(amount: Amount) -> (MintNode<SimpleKeyManager>, bls_dkg::outcome::Outcome, Dbc) {
     let genesis_owner = bls_dkg_id();
 
     let key_manager = SimpleKeyManager::new(
@@ -35,25 +29,25 @@ fn genesis(
         ),
         genesis_owner.public_key_set.public_key(),
     );
-    let mut genesis_node = Mint::new(key_manager, SimpleSpendBook::new());
+    let mut genesis_node = MintNode::new(key_manager);
 
-    let (content, transaction, (mint_key_set, mint_sig_share)) =
-        genesis_node.issue_genesis_dbc(amount).unwrap();
+    let genesis = genesis_node.issue_genesis_dbc(amount).unwrap();
 
-    let mint_sig = mint_key_set
-        .combine_signatures(vec![mint_sig_share.threshold_crypto()])
+    let mint_sig = genesis
+        .public_key_set
+        .combine_signatures(vec![genesis.transaction_sig.threshold_crypto()])
         .unwrap();
 
-    let transaction_sigs = BTreeMap::from_iter(
-        transaction
-            .inputs
-            .iter()
-            .map(|in_hash| (*in_hash, (mint_key_set.public_key(), mint_sig.clone()))),
-    );
+    let transaction_sigs = BTreeMap::from_iter(genesis.transaction.inputs.iter().map(|in_hash| {
+        (
+            *in_hash,
+            (genesis.public_key_set.public_key(), mint_sig.clone()),
+        )
+    }));
 
     let genesis_dbc = Dbc {
-        content,
-        transaction,
+        content: genesis.dbc_content,
+        transaction: genesis.transaction,
         transaction_sigs,
     };
 
@@ -77,31 +71,40 @@ fn bench_reissue_1_to_100(c: &mut Criterion) {
         .build()
         .unwrap();
 
-    let sig_share = genesis_owner
-        .secret_key_share
-        .derive_child(&genesis_dbc.spend_key_index())
-        .sign(&reissue_tx.blinded().hash());
-
-    let sig = genesis_owner
+    let spent_sig = genesis_owner
         .public_key_set
-        .combine_signatures(vec![(0, &sig_share)])
+        .combine_signatures([(
+            genesis_owner.index,
+            genesis_owner
+                .secret_key_share
+                .derive_child(&genesis_dbc.spend_key_index())
+                .sign(reissue_tx.blinded().hash()),
+        )])
+        .unwrap();
+    let spentbook_pks = genesis.key_manager.public_key_set().unwrap();
+    let spentbook_sig_share = genesis
+        .key_manager
+        .sign(&SpentProof::proof_msg(
+            &reissue_tx.blinded().hash(),
+            &spent_sig,
+        ))
         .unwrap();
 
-    let reissue = ReissueRequest {
-        transaction: reissue_tx,
-        input_ownership_proofs: HashMap::from_iter([(genesis_dbc.spend_key(), sig)]),
-    };
+    let rr = ReissueRequestBuilder::new(reissue_tx)
+        .add_spent_proof_share(
+            genesis_dbc.spend_key(),
+            SpentProofShare {
+                spent_sig,
+                spentbook_pks,
+                spentbook_sig_share,
+            },
+        )
+        .build()
+        .unwrap();
 
-    let spendbook = genesis.snapshot_spendbook();
     c.bench_function(&format!("reissue split 1 to {}", n_outputs), |b| {
         b.iter(|| {
-            genesis.reset_spendbook(spendbook.clone());
-            genesis
-                .reissue(
-                    black_box(reissue.clone()),
-                    black_box(BTreeSet::from_iter([genesis_dbc.spend_key()])),
-                )
-                .unwrap();
+            genesis.reissue(black_box(rr.clone())).unwrap();
         })
     });
 }
@@ -132,30 +135,39 @@ fn bench_reissue_100_to_1(c: &mut Criterion) {
         (out_dbc.owner, owner)
     }));
 
-    let sig_share = genesis_owner
-        .secret_key_share
-        .derive_child(&genesis_dbc.spend_key_index())
-        .sign(&reissue_tx.blinded().hash());
-
-    let sig = genesis_owner
+    let spent_sig = genesis_owner
         .public_key_set
-        .combine_signatures(vec![(0, &sig_share)])
+        .combine_signatures([(
+            genesis_owner.index,
+            genesis_owner
+                .secret_key_share
+                .derive_child(&genesis_dbc.spend_key_index())
+                .sign(reissue_tx.blinded().hash()),
+        )])
+        .unwrap();
+    let spentbook_pks = genesis.key_manager.public_key_set().unwrap();
+    let spentbook_sig_share = genesis
+        .key_manager
+        .sign(&SpentProof::proof_msg(
+            &reissue_tx.blinded().hash(),
+            &spent_sig,
+        ))
         .unwrap();
 
-    let input_ownership_proofs = HashMap::from_iter([(genesis_dbc.spend_key(), sig)]);
-
-    let reissue = ReissueRequest {
-        transaction: reissue_tx,
-        input_ownership_proofs,
-    };
-
-    let reissue_share = genesis
-        .reissue(
-            reissue.clone(),
-            BTreeSet::from_iter([genesis_dbc.spend_key()]),
+    let rr = ReissueRequestBuilder::new(reissue_tx)
+        .add_spent_proof_share(
+            genesis_dbc.spend_key(),
+            SpentProofShare {
+                spent_sig,
+                spentbook_pks,
+                spentbook_sig_share,
+            },
         )
+        .build()
         .unwrap();
 
+    let reissue_share = genesis.reissue(rr.clone()).unwrap();
+
     let (mint_key_set, mint_sig_share) = reissue_share
         .mint_node_signatures
         .values()
@@ -168,7 +180,7 @@ fn bench_reissue_100_to_1(c: &mut Criterion) {
         .combine_signatures(vec![mint_sig_share.threshold_crypto()])
         .unwrap();
 
-    let dbcs = Vec::from_iter(reissue.transaction.outputs.into_iter().map(|content| Dbc {
+    let dbcs = Vec::from_iter(rr.transaction.outputs.into_iter().map(|content| Dbc {
         content,
         transaction: reissue_share.dbc_transaction.clone(),
         transaction_sigs: BTreeMap::from_iter([(
@@ -191,32 +203,44 @@ fn bench_reissue_100_to_1(c: &mut Criterion) {
         .build()
         .unwrap();
 
-    let input_ownership_proofs = HashMap::from_iter(dbcs.iter().map(|dbc| {
+    let mut rr_builder = ReissueRequestBuilder::new(merge_tx.clone());
+
+    for dbc in dbcs.iter() {
         let owner = &dbc_owners[&dbc.owner()];
-        let sig_share = owner
-            .secret_key_share
-            .derive_child(&dbc.spend_key_index())
-            .sign(merge_tx.blinded().hash());
-        let sig = owner
+        let spent_sig = owner
             .public_key_set
-            .combine_signatures(vec![(0, &sig_share)])
+            .combine_signatures([(
+                owner.index,
+                owner
+                    .secret_key_share
+                    .derive_child(&dbc.spend_key_index())
+                    .sign(&merge_tx.blinded().hash()),
+            )])
+            .unwrap();
+        let spentbook_pks = genesis.key_manager.public_key_set().unwrap();
+        let spentbook_sig_share = genesis
+            .key_manager
+            .sign(&SpentProof::proof_msg(
+                &merge_tx.blinded().hash(),
+                &spent_sig,
+            ))
             .unwrap();
-        (dbc.spend_key(), sig)
-    }));
 
-    let merge_reissue = ReissueRequest {
-        transaction: merge_tx,
-        input_ownership_proofs,
-    };
-    let inputs = merge_reissue.transaction.blinded().inputs;
+        rr_builder = rr_builder.add_spent_proof_share(
+            dbc.spend_key(),
+            SpentProofShare {
+                spent_sig,
+                spentbook_pks,
+                spentbook_sig_share,
+            },
+        )
+    }
+
+    let merge_rr = rr_builder.build().unwrap();
 
-    let spendbook = genesis.snapshot_spendbook();
     c.bench_function(&format!("reissue merge {} to 1", n_outputs), |b| {
         b.iter(|| {
-            genesis.reset_spendbook(spendbook.clone());
-            genesis
-                .reissue(black_box(merge_reissue.clone()), black_box(inputs.clone()))
-                .unwrap();
+            genesis.reissue(black_box(merge_rr.clone())).unwrap();
         })
     });
 }
