@@ -13,16 +13,18 @@ use std::fmt;
 mod builder;
 mod dbc;
 mod dbc_content;
+mod dbc_packet;
 mod dbc_transaction;
 mod error;
 mod key_manager;
 mod mint;
-mod spend_book;
+mod spent_proof;
 
 pub use crate::{
-    builder::{DbcBuilder, Output, TransactionBuilder},
+    builder::{DbcBuilder, Output, ReissueRequestBuilder, TransactionBuilder},
     dbc::Dbc,
     dbc_content::{Amount, AmountSecrets, DbcContent},
+    dbc_packet::{DbcPacket, DerivedKeySet},
     dbc_transaction::DbcTransaction,
     error::{Error, Result},
     key_manager::{
@@ -30,10 +32,10 @@ pub use crate::{
         SimpleSigner,
     },
     mint::{
-        genesis_dbc_input, Mint, MintNodeSignatures, ReissueRequest, ReissueShare,
-        ReissueTransaction,
+        genesis_dbc_input, GenesisDbcShare, MintNode, MintNodeSignatures, ReissueRequest,
+        ReissueShare, ReissueTransaction,
     },
-    spend_book::{SimpleSpendBook, SpendBook, SpendKey},
+    spent_proof::{SpendKey, SpentProof, SpentProofShare},
 };
 
 #[derive(Clone, Copy, PartialEq, Eq, Hash, PartialOrd, Ord, Serialize, Deserialize)]
